/* gcm_alt.h with dummy types for MBEDTLS_GCM_ALT */
/*
 *  Copyright The Mbed TLS Contributors
 *  SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 */

#ifndef GCM_ALT_H
#define GCM_ALT_H

typedef struct mbedtls_gcm_context {
    int dummy;
}
mbedtls_gcm_context;


#endif /* gcm_alt.h */
