/* cmac_alt.h with dummy types for MBEDTLS_CMAC_ALT */
/*
 *  Copyright The Mbed TLS Contributors
 *  SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 */

#ifndef CMAC_ALT_H
#define CMAC_ALT_H

struct mbedtls_cmac_context_t {
    int dummy;
};


#endif /* cmac_alt.h */
