/* chachapoly_alt.h with dummy types for MBEDTLS_CHACHAPOLY_ALT */
/*
 *  Copyright The Mbed TLS Contributors
 *  SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 */

#ifndef CHACHAPOLY_ALT_H
#define CHACHAPOLY_ALT_H

#include "mbedtls/chacha20.h"

typedef struct mbedtls_chachapoly_context {
    int dummy;
}
mbedtls_chachapoly_context;


#endif /* chachapoly_alt.h */
