/*
 * ModSecurity, http://www.modsecurity.org/
 * Copyright (c) 2015 - 2023 Trustwave Holdings, Inc. (http://www.trustwave.com/)
 *
 * You may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * If any of the files related to licensing are missing or if you have any
 * other questions related to licensing please contact Trustwave Holdings, Inc.
 * directly using the email address security@modsecurity.org.
 *
 */

#include "remove_whitespace.h"

#include "remove_nulls.h"

namespace modsecurity::actions::transformations {


bool RemoveWhitespace::transform(std::string &value, const Transaction *trans) const {
    const char nonBreakingSpaces = 0xa0;
    const char nonBreakingSpaces2 = 0xc2;

    auto pred = [](const auto c) {
        // remove whitespaces and non breaking spaces (NBSP)
        return std::isspace(static_cast<unsigned char>(c))
            || c == nonBreakingSpaces
            || c == nonBreakingSpaces2;
    };

    return RemoveNulls::remove_if(value, pred);
}


}  // namespace modsecurity::actions::transformations

