/* ecjpake_alt.h with dummy types for MBEDTLS_ECJPAKE_ALT */
/*
 *  Copyright The Mbed TLS Contributors
 *  SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 */

#ifndef ECJPAKE_ALT_H
#define ECJPAKE_ALT_H

typedef struct mbedtls_ecjpake_context {
    int dummy;
} mbedtls_ecjpake_context;


#endif /* ecjpake_alt.h */
