/* camellia_alt.h with dummy types for MBEDTLS_CAMELLIA_ALT */
/*
 *  Copyright The Mbed TLS Contributors
 *  SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 */

#ifndef CAMELLIA_ALT_H
#define CAMELLIA_ALT_H

typedef struct mbedtls_camellia_context {
    int dummy;
}
mbedtls_camellia_context;


#endif /* camellia_alt.h */
