/* chacha20_alt.h with dummy types for MBEDTLS_CHACHA20_ALT */
/*
 *  Copyright The Mbed TLS Contributors
 *  SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 */

#ifndef CHACHA20_ALT_H
#define CHACHA20_ALT_H

typedef struct mbedtls_chacha20_context {
    int dummy;
}
mbedtls_chacha20_context;


#endif /* chacha20_alt.h */
