/* timing_alt.h with dummy types for MBEDTLS_TIMING_ALT */
/*
 *  Copyright The Mbed TLS Contributors
 *  SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 */

#ifndef TIMING_ALT_H
#define TIMING_ALT_H

struct mbedtls_timing_hr_time {
    int dummy;
};

typedef struct mbedtls_timing_delay_context {
    int dummy;
} mbedtls_timing_delay_context;


#endif /* timing_alt.h */
