/* des_alt.h with dummy types for MBEDTLS_DES_ALT */
/*
 *  Copyright The Mbed TLS Contributors
 *  SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 *
 */

#ifndef DES_ALT_H
#define DES_ALT_H

typedef struct mbedtls_des_context {
    int dummy;
}
mbedtls_des_context;

typedef struct mbedtls_des3_context {
    int dummy;
}
mbedtls_des3_context;


#endif /* des_alt.h */
