/* nist_kw_alt.h with dummy types for MBEDTLS_NIST_KW_ALT */
/*
 *  Copyright The Mbed TLS Contributors
 *  SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 */

#ifndef NIST_KW_ALT_H
#define NIST_KW_ALT_H

typedef struct {
    int dummy;
} mbedtls_nist_kw_context;


#endif /* nist_kw_alt.h */
